@@ -32,7 +32,7 @@ from chia.types.unfinished_block import UnfinishedBlock
 from chia.types.unfinished_header_block import UnfinishedHeaderBlock
 from chia.types.weight_proof import SubEpochChallengeSegment
 from chia.util.errors import Err
-from chia.util.generator_tools import get_block_header, block_removals_and_additions
+from chia.util.generator_tools import get_block_header, tx_removals_and_additions
 from chia.util.ints import uint16, uint32, uint64, uint128
 from chia.util.streamable import recurse_jsonify
 
@@ -162,6 +162,7 @@ class Blockchain(BlockchainInterface):
         block: FullBlock,
         pre_validation_result: Optional[PreValidationResult] = None,
         fork_point_with_peak: Optional[uint32] = None,
+        summaries_to_check: List[SubEpochSummary] = None,  # passed only on long sync
     ) -> Tuple[ReceiveBlockResult, Optional[Err], Optional[uint32]]:
         """
         This method must be called under the blockchain lock
@@ -205,10 +206,10 @@ class Blockchain(BlockchainInterface):
                     npc_result = get_name_puzzle_conditions(
                         block_generator, min(self.constants.MAX_BLOCK_COST_CLVM, block.transactions_info.cost), False
                     )
-                    removals, additions = block_removals_and_additions(block, npc_result.npc_list)
+                    removals, tx_additions = tx_removals_and_additions(npc_result.npc_list)
                 else:
-                    removals, additions = [], list(block.get_included_reward_coins())
-                header_block = get_block_header(block, additions, removals)
+                    removals, tx_additions = [], []
+                header_block = get_block_header(block, tx_additions, removals)
             else:
                 npc_result = None
                 header_block = get_block_header(block, [], [])
@@ -254,12 +255,15 @@ class Blockchain(BlockchainInterface):
         # Always add the block to the database
         async with self.block_store.db_wrapper.lock:
             try:
+                # Perform the DB operations to update the state, and rollback if something goes wrong
                 await self.block_store.db_wrapper.begin_transaction()
                 await self.block_store.add_full_block(block, block_record)
                 fork_height, peak_height, records = await self._reconsider_peak(
                     block_record, genesis, fork_point_with_peak, npc_result
                 )
                 await self.block_store.db_wrapper.commit_transaction()
+
+                # Then update the memory cache. It is important that this task is not cancelled and does not throw
                 self.add_block_record(block_record)
                 for fetched_block_record in records:
                     self.__height_to_hash[fetched_block_record.height] = fetched_block_record.header_hash
@@ -269,6 +273,7 @@ class Blockchain(BlockchainInterface):
                         ] = fetched_block_record.sub_epoch_summary_included
                 if peak_height is not None:
                     self._peak_height = peak_height
+                self.block_store.cache_block(block)
             except BaseException:
                 await self.block_store.db_wrapper.rollback_transaction()
                 raise
@@ -296,14 +301,11 @@ class Blockchain(BlockchainInterface):
                 block: Optional[FullBlock] = await self.block_store.get_full_block(block_record.header_hash)
                 assert block is not None
 
-                # Begins a transaction, because we want to ensure that the coin store and block store are only updated
-                # in sync.
                 if npc_result is not None:
-                    removals, additions = block_removals_and_additions(block, npc_result.npc_list)
+                    tx_removals, tx_additions = tx_removals_and_additions(npc_result.npc_list)
                 else:
-                    removals = []
-                    additions = list(block.get_included_reward_coins())
-                await self.coin_store.new_block(block, additions, removals)
+                    tx_removals, tx_additions = [], []
+                await self.coin_store.new_block(block, tx_additions, tx_removals)
                 await self.block_store.set_peak(block.header_hash)
                 return uint32(0), uint32(0), [block_record]
             return None, None, []
@@ -312,12 +314,13 @@ class Blockchain(BlockchainInterface):
         if block_record.weight > peak.weight:
             # Find the fork. if the block is just being appended, it will return the peak
             # If no blocks in common, returns -1, and reverts all blocks
-            if fork_point_with_peak is not None:
-                fork_height: int = fork_point_with_peak
+            if block_record.prev_hash == peak.header_hash:
+                fork_height: int = peak.height
+            elif fork_point_with_peak is not None:
+                fork_height = fork_point_with_peak
             else:
                 fork_height = find_fork_point_in_chain(self, block_record, peak)
 
-            # Rollback to fork
             if block_record.prev_hash != peak.header_hash:
                 await self.coin_store.rollback_to_block(fork_height)
             # Rollback sub_epoch_summaries
@@ -349,10 +352,12 @@ class Blockchain(BlockchainInterface):
                 records_to_add.append(fetched_block_record)
                 if fetched_block_record.is_transaction_block:
                     if fetched_block_record.header_hash == block_record.header_hash:
-                        removals, additions = await self.get_removals_and_additions(fetched_full_block, npc_result)
+                        tx_removals, tx_additions = await self.get_tx_removals_and_additions(
+                            fetched_full_block, npc_result
+                        )
                     else:
-                        removals, additions = await self.get_removals_and_additions(fetched_full_block, None)
-                    await self.coin_store.new_block(fetched_full_block, additions, removals)
+                        tx_removals, tx_additions = await self.get_tx_removals_and_additions(fetched_full_block, None)
+                    await self.coin_store.new_block(fetched_full_block, tx_additions, tx_removals)
 
             # Changes the peak to be the new peak
             await self.block_store.set_peak(block_record.header_hash)
@@ -361,7 +366,7 @@ class Blockchain(BlockchainInterface):
         # This is not a heavier block than the heaviest we have seen, so we don't change the coin set
         return None, None, []
 
-    async def get_removals_and_additions(
+    async def get_tx_removals_and_additions(
         self, block: FullBlock, npc_result: Optional[NPCResult] = None
     ) -> Tuple[List[bytes32], List[Coin]]:
         if block.is_transaction_block():
@@ -370,10 +375,10 @@ class Blockchain(BlockchainInterface):
                     block_generator: Optional[BlockGenerator] = await self.get_block_generator(block)
                     assert block_generator is not None
                     npc_result = get_name_puzzle_conditions(block_generator, self.constants.MAX_BLOCK_COST_CLVM, False)
-                removals, additions = block_removals_and_additions(block, npc_result.npc_list)
-                return removals, additions
+                tx_removals, tx_additions = tx_removals_and_additions(npc_result.npc_list)
+                return tx_removals, tx_additions
             else:
-                return [], list(block.get_included_reward_coins())
+                return [], []
         else:
             return [], []
 
@@ -639,16 +644,25 @@ class Blockchain(BlockchainInterface):
                 header_hash: bytes32 = self.height_to_hash(uint32(height))
                 hashes.append(header_hash)
 
-        blocks: List[FullBlock] = await self.block_store.get_blocks_by_hash(hashes)
+        blocks: List[FullBlock] = []
+        for hash in hashes.copy():
+            block = self.block_store.block_cache.get(hash)
+            if block is not None:
+                blocks.append(block)
+                hashes.remove(hash)
+        blocks_on_disk: List[FullBlock] = await self.block_store.get_blocks_by_hash(hashes)
+        blocks.extend(blocks_on_disk)
         header_blocks: Dict[bytes32, HeaderBlock] = {}
 
         for block in blocks:
             if self.height_to_hash(block.height) != block.header_hash:
                 raise ValueError(f"Block at {block.header_hash} is no longer in the blockchain (it's in a fork)")
-            additions: List[CoinRecord] = await self.coin_store.get_coins_added_at_height(block.height)
+            tx_additions: List[CoinRecord] = [
+                c for c in (await self.coin_store.get_coins_added_at_height(block.height)) if not c.coinbase
+            ]
             removed: List[CoinRecord] = await self.coin_store.get_coins_removed_at_height(block.height)
             header = get_block_header(
-                block, [record.coin for record in additions], [record.coin.name() for record in removed]
+                block, [record.coin for record in tx_additions], [record.coin.name() for record in removed]
             )
             header_blocks[header.header_hash] = header
 
