@@ -96,11 +96,11 @@ class MempoolManager:
         additions = []
         broke_from_inner_loop = False
         log.info(f"Starting to make block, max cost: {self.constants.MAX_BLOCK_COST_CLVM}")
-        for dic in self.mempool.sorted_spends.values():
+        for dic in reversed(self.mempool.sorted_spends.values()):
             if broke_from_inner_loop:
                 break
             for item in dic.values():
-                log.info(f"Cumulative cost: {cost_sum}")
+                log.info(f"Cumulative cost: {cost_sum}, fee per cost: {item.fee / item.cost}")
                 if (
                     item.cost + cost_sum <= self.limit_factor * self.constants.MAX_BLOCK_COST_CLVM
                     and item.fee + fee_sum <= self.constants.MAX_COIN_AMOUNT
@@ -157,7 +157,7 @@ class MempoolManager:
             self.seen_bundle_hashes.pop(first_in)
 
     def seen(self, bundle_hash: bytes32) -> bool:
-        """Return true if we saw this spendbundle before"""
+        """Return true if we saw this spendbundle recently"""
         return bundle_hash in self.seen_bundle_hashes
 
     def remove_seen(self, bundle_hash: bytes32):
@@ -229,8 +229,8 @@ class MempoolManager:
         program: Optional[SerializedProgram] = None,
     ) -> Tuple[Optional[uint64], MempoolInclusionStatus, Optional[Err]]:
         """
-        Tries to add spendbundle to either self.mempools or to_pool if it's specified.
-        Returns true if it's added in any of pools, Returns error if it fails.
+        Tries to add spend bundle to the mempool
+        Returns the cost (if SUCCESS), the result (MempoolInclusion status), and an optional error
         """
         start_time = time.time()
         if self.peak is None:
@@ -505,7 +505,14 @@ class MempoolManager:
         self.mempool = Mempool(self.mempool_max_total_cost)
 
         for item in old_pool.spends.values():
-            await self.add_spendbundle(item.spend_bundle, item.npc_result, item.spend_bundle_name, False, item.program)
+            _, result, _ = await self.add_spendbundle(
+                item.spend_bundle, item.npc_result, item.spend_bundle_name, False, item.program
+            )
+            # If the spend bundle was confirmed or conflicting (can no longer be in mempool), it won't be successfully
+            # added to the new mempool. In this case, remove it from seen, so in the case of a reorg, it can be
+            # resubmitted
+            if result != MempoolInclusionStatus.SUCCESS:
+                self.remove_seen(item.spend_bundle_name)
 
         potential_txs_copy = self.potential_txs.copy()
         self.potential_txs = {}
