@@ -1,6 +1,6 @@
 import asyncio
 from ssl import SSLContext
-from typing import Dict, List, Optional
+from typing import Dict, List, Optional, Any
 
 import aiohttp
 
@@ -37,7 +37,7 @@ class RpcClient:
         self.closing_task = None
         return self
 
-    async def fetch(self, path, request_json):
+    async def fetch(self, path, request_json) -> Any:
         async with self.session.post(self.url + path, json=request_json, ssl_context=self.ssl_context) as response:
             response.raise_for_status()
             res_json = await response.json()
